"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Bluebird = require("bluebird");
const chalk_1 = require("chalk");
const ora = require("ora");
const pad = require("pad");
const semver = require("semver");
const clients_1 = require("../../clients");
const conf_1 = require("../../conf");
const logger_1 = require("../../logger");
const prompts_1 = require("../prompts");
const utils_1 = require("./utils");
const { listAvailableServices, listInstalledServices, installService } = clients_1.router;
const promptUpdate = () => Promise.resolve(prompts_1.promptConfirm('Apply version updates?'));
const calculateColSize = (names) => Math.max(...names.map(n => n.length));
const logLatest = (name, version, colSize) => console.log(`${pad(name, colSize)}  ${chalk_1.default.yellow(version)}`);
const logUpdate = (name, currentVersion, latestVersion, colSize) => {
    const [fromVersion, toVersion] = utils_1.diffVersions(currentVersion, latestVersion);
    console.log(`${pad(name, colSize)}  ${fromVersion} ${chalk_1.default.gray('->')} ${toVersion}`);
};
const logVersionMap = ({ latest, update }) => {
    const latestKeys = Object.keys(latest);
    const updateKeys = Object.keys(update);
    const colSize = calculateColSize([...latestKeys, ...updateKeys]);
    latestKeys.map(k => logLatest(k, latest[k], colSize));
    updateKeys.map(k => logUpdate(k, update[k].current, update[k].latest, colSize));
};
const createVersionMap = (availableRes, installedRes) => installedRes.reduce((acc, { name, version: currentVersion }) => {
    const tag = utils_1.getTag(currentVersion);
    const latestVersion = availableRes[name].versions[conf_1.Region.Production] // See comment in src/modules/infra/install.ts:82
        .filter(v => utils_1.getTag(v) === tag)
        .sort(semver.rcompare)[0];
    if (currentVersion !== latestVersion) {
        acc.update[name] = {
            current: currentVersion,
            latest: latestVersion,
        };
    }
    else {
        acc.latest[name] = currentVersion;
    }
    return acc;
}, { latest: {}, update: {} });
const hasUpdate = (update) => Object.keys(update).length > 0;
const installUpdates = (update) => Bluebird.all(Object.keys(update).map(name => installService(name, update[name].latest)));
exports.default = () => {
    const spinner = ora('Getting available updates').start();
    return Promise.all([listAvailableServices(), listInstalledServices()])
        .tap(() => spinner.stop())
        .spread(createVersionMap)
        .tap(logVersionMap)
        .then(({ update }) => {
        console.log('');
        return hasUpdate(update)
            ? promptUpdate()
                .then(confirm => {
                if (!confirm) {
                    return;
                }
                spinner.text = 'Installing';
                spinner.start();
                return installUpdates(update);
            })
                .then(() => spinner.stop())
                .then(() => logger_1.default.info('All updates were installed'))
            : logger_1.default.info('All up to date!');
    })
        .catch(err => {
        spinner.stop();
        throw err;
    });
};
