"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const retry = require("async-retry");
const axios_1 = require("axios");
const child_process_es6_promise_1 = require("child-process-es6-promise");
const fs_extra_1 = require("fs-extra");
const path_1 = require("path");
const ramda_1 = require("ramda");
const conf_1 = require("../conf");
const conf_2 = require("../conf");
const env_1 = require("../env");
const logger_1 = require("../logger");
const manifest_1 = require("../manifest");
const utils_1 = require("./apps/utils");
const utils_2 = require("./utils");
const account = conf_1.getAccount();
const workspace = conf_1.getWorkspace();
const root = manifest_1.getAppRoot();
const builderHubTimeout = 2000; // 2 seconds
const buildersToAddAdditionalPackages = ['react', 'node'];
const addToPackageJson = {
    'eslint': '^5.15.1',
    'eslint-config-vtex': '^10.1.0',
    'eslint-config-vtex-react': '^4.1.0',
};
const addToEslintrc = {
    'react': {
        'extends': 'vtex-react',
        'env': {
            'browser': true,
            'es6': true,
            'jest': true,
        },
    },
    'node': {
        'extends': 'vtex',
        'env': {
            'node': true,
            'es6': true,
            'jest': true,
        },
    },
};
const typingsPath = 'public/_types';
const typingsURLRegex = /_v\/\w*\/typings/;
const getVendor = (appId) => appId.split('.')[0];
const builderHttp = axios_1.default.create({
    baseURL: `http://builder-hub.vtex.${env_1.region()}.vtex.io/${account}/${workspace}`,
    timeout: builderHubTimeout,
    headers: {
        'Authorization': conf_2.getToken(),
    },
});
const retryOpts = {
    retries: 2,
    minTimeout: 1000,
    factor: 2,
};
const resolvePackageJsonPath = (builder) => path_1.resolve(root, `${builder}/package.json`);
const resolveTSConfigPath = (builder) => path_1.resolve(root, `${builder}/tsconfig.json`);
const resolveESLintPath = (builder) => path_1.resolve(root, `${builder}/.eslintrc`);
const typingsInfo = () => __awaiter(this, void 0, void 0, function* () {
    const getTypingsInfo = (_, tryCount) => __awaiter(this, void 0, void 0, function* () {
        if (tryCount > 1) {
            logger_1.default.info(`Retrying...${tryCount - 1} (get typings info from builder-hub)`);
        }
        try {
            const res = yield builderHttp.get(`/_v/builder/0/typings`);
            return res.data.typingsInfo;
        }
        catch (err) {
            const statusMessage = err.response.status ?
                `: Status ${err.response.status}` : '';
            logger_1.default.error(`Error fetching typings info ${statusMessage} (try: ${tryCount})`);
            throw err;
        }
    });
    try {
        return retry(getTypingsInfo, retryOpts);
    }
    catch (e) {
        logger_1.default.error('Unable to get typings info from vtex.builder-hub.');
        return {};
    }
});
const appTypingsURL = (appName, appVersion, builder) => __awaiter(this, void 0, void 0, function* () {
    const appId = yield utils_1.resolveAppId(appName, appVersion);
    const vendor = getVendor(appId);
    if (utils_1.isLinked({ 'version': appId, vendor, name: '', builders: {} })) {
        return `https://${workspace}--${account}.${env_1.publicEndpoint()}/_v/private/typings/linked/v1/${appId}/${typingsPath}/${builder}`;
    }
    return `http://${vendor}.vteximg.com.br/_v/public/typings/v1/${appId}/${typingsPath}/${builder}`;
});
const appsWithTypingsURLs = (builder, appDependencies) => __awaiter(this, void 0, void 0, function* () {
    const result = {};
    const appNamesAndDependencies = ramda_1.toPairs(appDependencies);
    yield Promise.map(appNamesAndDependencies, ([appName, appVersion]) => __awaiter(this, void 0, void 0, function* () {
        try {
            result[appName] = yield appTypingsURL(appName, appVersion, builder);
        }
        catch (e) {
            logger_1.default.error(`Unable to generate typings URL for ${appName}@${appVersion}.`);
        }
    }));
    return result;
});
const yarnAddESLint = (relativePath) => {
    logger_1.default.info(`Adding lint configs in ${relativePath}`);
    const lintDeps = ramda_1.join(' ', ramda_1.values(ramda_1.mapObjIndexed((version, name) => `${name}@${version}`, addToPackageJson)));
    child_process_es6_promise_1.execSync(`${utils_2.yarnPath} add ${lintDeps} --dev`, { stdio: 'inherit', cwd: path_1.resolve(root, `${relativePath}`) });
};
const getInjectedDeps = (typingsData, version, builder) => {
    const builderTypingsData = ramda_1.prop(builder, typingsData);
    if (builderTypingsData && ramda_1.has(version, builderTypingsData)) {
        return ramda_1.path([version, 'injectedDependencies'], builderTypingsData);
    }
    return null;
};
const injectTypingsInPackageJson = (appDeps, builder) => __awaiter(this, void 0, void 0, function* () {
    const packageJsonPath = resolvePackageJsonPath(builder);
    if (yield fs_extra_1.pathExists(packageJsonPath)) {
        const packageJson = fs_extra_1.readJsonSync(packageJsonPath);
        const oldDevDeps = packageJson.devDependencies || {};
        const oldTypingsEntries = ramda_1.filter(ramda_1.test(typingsURLRegex), oldDevDeps);
        const newTypingsEntries = yield appsWithTypingsURLs(builder, appDeps);
        if (!ramda_1.equals(oldTypingsEntries, newTypingsEntries)) {
            const cleanOldDevDeps = ramda_1.reject(ramda_1.test(typingsURLRegex), oldDevDeps);
            fs_extra_1.outputJsonSync(packageJsonPath, Object.assign({}, packageJson, { 'devDependencies': Object.assign({}, cleanOldDevDeps, newTypingsEntries) }), { spaces: 2 });
            try {
                utils_2.runYarn(builder, true);
            }
            catch (e) {
                logger_1.default.error(`Error running Yarn in ${builder}.`);
                yield fs_extra_1.outputJsonSync(packageJsonPath, packageJson, { spaces: 2 }); // Revert package.json to original state.
            }
        }
    }
});
const getTypings = (manifest) => __awaiter(this, void 0, void 0, function* () {
    const typingsData = yield typingsInfo();
    const buildersWithInjectedDeps = ramda_1.pipe(ramda_1.prop('builders'), ramda_1.mapObjIndexed(ramda_1.curry(getInjectedDeps)(typingsData)), ramda_1.reject(ramda_1.isNil))(manifest);
    const buildersWithAppDeps = ramda_1.pipe(ramda_1.mapObjIndexed((injectedDependencies) => ramda_1.merge(manifest.dependencies, injectedDependencies)), ramda_1.reject(ramda_1.isEmpty))(buildersWithInjectedDeps);
    yield ramda_1.pipe(ramda_1.mapObjIndexed(injectTypingsInPackageJson), ramda_1.values, Promise.all)(buildersWithAppDeps);
});
const getTSConfigFromBuilderHub = () => __awaiter(this, void 0, void 0, function* () {
    const downloadTSConfig = (_, tryCount) => __awaiter(this, void 0, void 0, function* () {
        if (tryCount > 1) {
            logger_1.default.info(`Retrying...${tryCount - 1} (get tsconfig from builder-hub)`);
        }
        try {
            const res = yield builderHttp.get(`/_v/builder/0/tsconfig`);
            return res.data;
        }
        catch (err) {
            const statusMessage = err.response.status ?
                `: Status ${err.response.status}` : '';
            logger_1.default.error(`Error fetching tsconfig from builder-hub ${statusMessage} (try: ${tryCount})`);
            throw err;
        }
    });
    try {
        return retry(downloadTSConfig, retryOpts);
    }
    catch (e) {
        logger_1.default.error('Unable to get tsconfig.json info from vtex.builder-hub.');
        return {};
    }
});
const selectTSConfig = (tsconfigsFromBuilder, version, builder) => {
    const builderTSConfig = ramda_1.prop(builder, tsconfigsFromBuilder);
    if (builderTSConfig && ramda_1.has(version, builderTSConfig)) {
        return ramda_1.prop(version, builderTSConfig);
    }
    return null;
};
const getTSConfig = (manifest) => __awaiter(this, void 0, void 0, function* () {
    const tsconfigsFromBuilder = yield getTSConfigFromBuilderHub();
    const buildersWithBaseTSConfig = ramda_1.compose(ramda_1.reject(ramda_1.isNil), ramda_1.mapObjIndexed(ramda_1.curry(selectTSConfig)(tsconfigsFromBuilder)), ramda_1.prop('builders'))(manifest);
    return ramda_1.mapObjIndexed((baseTSConfig, builder) => {
        try {
            const tsconfigPath = resolveTSConfigPath(builder);
            let currentTSConfig = {};
            try {
                currentTSConfig = fs_extra_1.readJsonSync(tsconfigPath);
            }
            catch (e) {
                if (e.code === 'ENOENT') {
                    logger_1.default.warn(`No tsconfig.json found in ${tsconfigPath}. Generating one...`);
                }
                else {
                    throw e;
                }
            }
            const newTSConfig = ramda_1.mergeDeepRight(currentTSConfig, baseTSConfig);
            fs_extra_1.outputJsonSync(tsconfigPath, newTSConfig, { spaces: 2 }); // Revert package.json to original state.
        }
        catch (e) {
            logger_1.default.error(e);
        }
    })(buildersWithBaseTSConfig);
});
const createESLintSetup = (lintDeps, builder) => __awaiter(this, void 0, void 0, function* () {
    try {
        const packageJsonPath = resolvePackageJsonPath(builder);
        const devDependencies = (ramda_1.prop('devDependencies', yield fs_extra_1.readJson(packageJsonPath))) || {};
        if (ramda_1.difference(lintDeps, ramda_1.intersection(lintDeps, ramda_1.keys(devDependencies))).length !== 0) {
            yarnAddESLint(builder);
        }
        yield fs_extra_1.outputJson(resolveESLintPath(builder), addToEslintrc[builder], { spaces: 2 });
    }
    catch (e) {
        logger_1.default.error(e);
    }
});
const setupESLint = (manifest) => __awaiter(this, void 0, void 0, function* () {
    const builders = ramda_1.keys(ramda_1.prop('builders', manifest) || {});
    const filteredBuilders = ramda_1.intersection(builders, buildersToAddAdditionalPackages);
    const lintDeps = ramda_1.keys(addToPackageJson);
    return ramda_1.compose(Promise.all, ramda_1.map(ramda_1.curry(createESLintSetup)(lintDeps)))(filteredBuilders);
});
exports.default = () => __awaiter(this, void 0, void 0, function* () {
    const manifest = yield manifest_1.getManifest();
    yield setupESLint(manifest);
    yield getTSConfig(manifest);
    yield getTypings(manifest);
});
